/* $Id: platform_win.c 7623 2013-05-06 12:48:48Z chris $ */

#include <assert.h>

#include "platform_win.h"
#include "my_ntddscsi.h"

#define var_offsetof(v, m)                      \
  ((size_t)((char *)&((v).m) - (char *)&(v)))

int open_device(const char *device_name, HANDLE *handle)
{
  HANDLE hVolume;
  UINT uDriveType;
  char szVolumeName[8];
  char szRootName[5];
  DWORD dwAccessFlags;

  if (device_name[0] == '\0' ||
      device_name[1] != ':' ||
      device_name[2] != '\0')
    return EINVAL;

  szRootName[0]=device_name[0];
  szRootName[1]=':';
  szRootName[2]='\\';
  szRootName[3]='\0';

  uDriveType = GetDriveType(szRootName);

  switch(uDriveType) {
  case DRIVE_CDROM:
    dwAccessFlags = GENERIC_READ | GENERIC_WRITE;
    break;

  default:
    return EINVAL;
  }

  szVolumeName[0]='\\';
  szVolumeName[1]='\\';
  szVolumeName[2]='.';
  szVolumeName[3]='\\';
  szVolumeName[4]=device_name[0];
  szVolumeName[5]=':';
  szVolumeName[6]='\0';

  hVolume = CreateFile(   szVolumeName,
                          dwAccessFlags,
                          FILE_SHARE_READ | FILE_SHARE_WRITE,
                          NULL,
                          OPEN_EXISTING,
                          FILE_ATTRIBUTE_NORMAL,
                          NULL );

  if (hVolume == INVALID_HANDLE_VALUE)
    return EINVAL;

  *handle = hVolume;

  return 0;
}

int close_device(HANDLE handle)
{
  BOOL success = CloseHandle(handle);
  return success ? 0 : EINVAL;
}

int scsi_cmd(HANDLE handle, uint8_t *cmd, size_t cmd_len, uint8_t *data, size_t data_len)
{
  SCSI_PASS_THROUGH_DIRECT sptd;
  DWORD dwBytesReturned;
  BOOL success;

  sptd.Length=sizeof(sptd);
  sptd.PathId=0;   //SCSI card ID will be filled in automatically
  sptd.TargetId=0; //SCSI target ID will also be filled in
  sptd.Lun=0;      //SCSI lun ID will also be filled in
  sptd.CdbLength = cmd_len;
  sptd.SenseInfoLength=0; //Don't return any sense data
  sptd.DataIn=SCSI_IOCTL_DATA_IN; //There will be data from drive
  sptd.DataTransferLength = data_len;
  sptd.TimeOutValue=60;  //SCSI timeout value (60 seconds - maybe it should be longer)
  sptd.DataBuffer=data;
  sptd.SenseInfoOffset=0;

  assert(cmd_len <= sizeof sptd.Cdb);
  memcpy(sptd.Cdb, cmd, cmd_len);

  //Send the command to drive
  success=DeviceIoControl(handle,
                          IOCTL_SCSI_PASS_THROUGH_DIRECT,
                          &sptd, sizeof sptd,
                          &sptd, sizeof sptd,
                          &dwBytesReturned,
                          NULL);

  if (!success ||
      (dwBytesReturned >= (var_offsetof(sptd, ScsiStatus)
                           + sizeof sptd.ScsiStatus) &&
      sptd.ScsiStatus != 0))
    return EIO;

  return 0;
}
